@@ -1,4 +1,4 @@
-# Copyright (c) 2017, Riverbank Computing Limited
+# Copyright (c) 2018, Riverbank Computing Limited
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -23,7 +23,7 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-# This is v2.3 of this boilerplate.
+# This is v2.4 of this boilerplate.
 
 
 from distutils import sysconfig
@@ -60,7 +60,10 @@ class ModuleConfiguration(object):
 
     # The version of the module as a string.  Set it to None if you don't
     # provide version information.
-    version = '2.10.3'
+    version = '2.10.6'
+
+    # The name of the PEP 376 .dist-info directory to be created.
+    distinfo_name = 'QScintilla'
 
     # Set if a configuration script is provided that handles versions of PyQt4
     # prior to v4.10 (i.e. versions where the pyqtconfig.py module is
@@ -730,6 +733,7 @@ class _TargetConfiguration:
         self.sip_version_str = None
         self.sysroot = ''
         self.stubs_dir = ''
+        self.distinfo = False
 
         self.prot_is_public = (self.py_platform.startswith('linux') or self.py_platform == 'darwin')
 
@@ -1081,6 +1085,9 @@ class _TargetConfiguration:
         if opts.sip is not None:
             self.sip = opts.sip
 
+        if pkg_config.distinfo_name and opts.distinfo:
+            self.distinfo = True
+
         pkg_config.apply_options(self, opts)
 
     @staticmethod
@@ -1121,6 +1128,11 @@ def _create_optparser(target_config, pkg_config):
             metavar="SPEC",
             help="pass -spec SPEC to qmake")
 
+    if pkg_config.distinfo_name:
+        p.add_option("--no-dist-info", action="store_false", default=True,
+                dest="distinfo",
+                help="do not install the dist-info directory")
+
     if _has_stubs(pkg_config):
         p.add_option('--stubsdir', dest='stubsdir', type='string',
                 default=None, action='callback',
@@ -1291,10 +1303,12 @@ def _inform_user(target_config, pkg_config):
                 os.path.join(target_config.api_dir, 'api', 'python'))
 
 
-def _generate_code(target_config, opts, pkg_config, module_config):
+def _generate_code(target_config, opts, pkg_config, module_config, all_installs):
     """ Generate the code for the module.  target_config is the target
     configuration.  opts are the command line options.  pkg_config is the
     package configuration.  module_config is the module configuration.
+    all_installs is a list that is updated with the files installed for this
+    module.
     """
 
     inform(
@@ -1380,7 +1394,7 @@ def _generate_code(target_config, opts, pkg_config, module_config):
         error("Unable to create the C++ code.")
 
     # Generate the .pro file.
-    _generate_pro(target_config, opts, module_config)
+    _generate_pro(target_config, opts, module_config, all_installs)
 
 
 def _get_qt_qmake_config(qmake_config, qt_version):
@@ -1406,10 +1420,11 @@ def _write_qt_qmake_config(qt_qmake_config, pro):
             pro.write('    %s += %s\n' % (name, value))
 
 
-def _generate_pro(target_config, opts, module_config):
+def _generate_pro(target_config, opts, module_config, all_installs):
     """ Generate the .pro file for the module.  target_config is the target
     configuration.  opts are the command line options.  module_config is the
-    module configuration.
+    module configuration.  all_installs is a list that is updated with the
+    files installed for this module.
     """
 
     inform("Generating the .pro file for the %s module..." % module_config.name)
@@ -1495,6 +1510,9 @@ target.path = %s
 INSTALLS += target
 ''' % quote(target_config.module_dir))
 
+    mod_ext = '.pyd' if sys.platform == 'win32' else '.so'
+    all_installs.append(target_config.module_dir + '/' + mname + mod_ext)
+
     sip_installs = module_config.get_sip_installs(target_config)
     if sip_installs is not None:
         path, files = sip_installs
@@ -1510,6 +1528,8 @@ sip.files =''' % quote(path))
 INSTALLS += sip
 ''')
 
+        all_installs.append(path)
+
     pro.write('\n')
 
     # These optimisations could apply to other platforms.
@@ -1768,8 +1788,11 @@ def _main(argv, pkg_config):
     pkg_config.pre_code_generation(target_config)
 
     # Generate the code.
+    all_installs = []
+
     for module_config in pkg_config.modules:
-        _generate_code(target_config, opts, pkg_config, module_config)
+        _generate_code(target_config, opts, pkg_config, module_config,
+                all_installs)
 
     # Concatenate any .api files.
     if pkg_config.qscintilla_api_file and target_config.api_dir != '':
@@ -1811,12 +1834,39 @@ pep484_stubs.files = %s
 INSTALLS += pep484_stubs
 ''' % (target_config.stubs_dir, ' '.join(stubs)))
 
+            all_installs.extend(
+                    [target_config.stubs_dir + '/' + pyi for pyi in stubs])
+
     if pkg_config.qscintilla_api_file and target_config.api_dir != '':
+        api_dir = target_config.api_dir + '/api/python'
+        api_file = pkg_config.qscintilla_api_file + '.api'
+
         pro.write('''
-api.path = %s/api/python
-api.files = %s.api
+api.path = %s
+api.files = %s
 INSTALLS += api
-''' % (target_config.api_dir, pkg_config.qscintilla_api_file))
+''' % (api_dir, api_file))
+
+        all_installs.append(api_dir + '/' + api_file)
+
+    if target_config.distinfo:
+        distinfo_dir = os.path.join(target_config.py_module_dir,
+            pkg_config.distinfo_name + '-' + pkg_config.version + '.dist-info')
+        mk_distinfo = sys.executable + ' mk_distinfo.py ' + distinfo_dir + ' installed.txt'
+
+        pro.write('''
+distinfo.extra = %s
+distinfo.path = %s
+INSTALLS += distinfo
+''' % (mk_distinfo, target_config.module_dir))
+
+        # Create the file containing the names of all installed files.
+        installed = open('installed.txt', 'w')
+
+        for install in all_installs:
+            installed.write(install + '\n')
+
+        installed.close()
 
     pro.close()
 
@@ -1836,11 +1886,17 @@ if __name__ == '__main__':
     except NameError:
         pkg_config_type = type('PackageConfiguration', (object, ), {})
 
-    if not hasattr(pkg_config_type, 'modules'):
+    if hasattr(pkg_config_type, 'modules'):
+        # Provide some default values.
+        pkg_config_type.distinfo_name = getattr(pkg_config_type,
+                'distinfo_name', '')
+    else:
         mod_config_type = ModuleConfiguration
 
         # Extract the package-specific attributes and methods.
         pkg_config_type.descriptive_name = mod_config_type.descriptive_name
+        pkg_config_type.distinfo_name = getattr(mod_config_type,
+                'distinfo_name', '')
         pkg_config_type.legacy_configuration_script = mod_config_type.legacy_configuration_script
         pkg_config_type.minimum_sip_version = mod_config_type.minimum_sip_version
         pkg_config_type.protected_is_public_is_supported = mod_config_type.protected_is_public_is_supported
